@@ -1,5 +1,5 @@
 /*
- * $Id: CimCurl.cpp,v 1.13 2009/03/04 19:10:54 tyreld Exp $
+ * $Id: CimCurl.cpp,v 1.18 2013/09/20 23:26:32 hellerda Exp $
  *
  * CimCurl.cpp
  *
@@ -28,10 +28,14 @@
 #endif
 
 #include "CimCurl.h"
+#include <unistd.h> // for getpass()
+#include <cerrno> 
 
 extern int useNl;
 extern int dumpXml;
 extern int reqChunking;
+extern int waitTime;
+extern int expect100;
 
 // These are the constant headers added to all requests
 static const char *headers[] = {
@@ -40,7 +44,6 @@ static const char *headers[] = {
     "CIMProtocolVersion: 1.0",
     "CIMOperation: MethodCall",
     "Accept:",
-    "Expect:",
 };
 #define NUM_HEADERS ((sizeof(headers))/(sizeof(headers[0])))
 
@@ -154,6 +157,15 @@ void CimomCurl::genRequest(URL &url, const char *op, bool cls, bool keys)
 
     /* Initialize curl with the url */
     rv = curl_easy_setopt(mHandle, CURLOPT_URL, mUri.c_str());
+    
+    /* Set IPv6 scope (zone) identifier if provided */
+    if (url.zone_id >= 0) {
+#if LIBCURL_VERSION_NUM >= 0x071300
+    rv = curl_easy_setopt(mHandle, CURLOPT_ADDRESS_SCOPE, url.zone_id);
+#else
+    throw URLException("IPv6 zone identifier requires libcurl 7.19 or greater");
+#endif
+    }
 
     /* Disable progress output */
     rv = curl_easy_setopt(mHandle, CURLOPT_NOPROGRESS, 1);
@@ -212,6 +224,10 @@ void CimomCurl::genRequest(URL &url, const char *op, bool cls, bool keys)
     object += sb;
     mHeaders = curl_slist_append(mHeaders, object.c_str());
 
+    // Optionally send "Expect: 100-continue" header
+    if (expect100)
+      mHeaders = curl_slist_append(mHeaders, "Expect: 100-continue");
+
     // Set all of the headers for the request
     rv = curl_easy_setopt(mHandle, CURLOPT_HTTPHEADER, mHeaders);
 
@@ -279,6 +295,17 @@ string CimomCurl::getResponse()
     if (response.length() == 0)
         throw HttpException("No data received from server.");
     
+    // This is not ideal as we have the response but wait to return it.
+    // However it is much simpler to implement this here.
+    if (waitTime > 0) {
+        printf("waiting %ds...\n", waitTime);
+        sleep(waitTime);
+    }
+    else if (waitTime < 0) {
+        waitTime = (int) -1;
+        printf("waiting forever...\n");
+        sleep(-1);
+    }
     return response;
 }
 
@@ -312,13 +339,21 @@ void CimomCurl::setClientCertificates(const char * cacert, int noverify,
        throw HttpException("Could not disable peer verification.");
      }
    } else if (cacert) {
-     if ((rv=curl_easy_setopt(mHandle,CURLOPT_SSL_VERIFYPEER,1))) {
-       cerr << getErrorMessage(rv) << endl;
-       throw HttpException("Could not enable peer verification.");
-     }
-     if ((rv=curl_easy_setopt(mHandle,CURLOPT_CAINFO,cacert))) {
-       cerr << getErrorMessage(rv) << endl;
-       throw HttpException("Could not load CA certificate.");
+     FILE *fp;
+     if ((fp = fopen(cacert, "r"))) {
+       if ((rv=curl_easy_setopt(mHandle,CURLOPT_SSL_VERIFYPEER,1))) {
+         cerr << getErrorMessage(rv) << endl;
+         throw HttpException("Could not enable peer verification.");
+       }
+       if ((rv=curl_easy_setopt(mHandle,CURLOPT_CAINFO,cacert))) {
+         cerr << getErrorMessage(rv) << endl;
+         throw HttpException("Could not load CA certificate.");
+       }
+       fclose(fp);
+     } else {
+       throw HttpException(
+           string("Could not open CA certificate file: ") + string(cacert)
+               + string(" (") + string(strerror(errno)) + string(")"));
      }
    } else {
      throw HttpException("Must either specify -noverify or -cacert for https URLs.");
