@@ -1,21 +1,24 @@
 /*
+ * $Id: CimXml.h,v 1.40 2013/03/25 04:05:49 hellerda Exp $
+ *
  * CimXml.h
  *
- * (C) Copyright IBM Corp. 2002, 2004
+ * © Copyright IBM Corp. 2002, 2004, 2006, 2007, 2008
  * 
- * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE
+ * THIS FILE IS PROVIDED UNDER THE TERMS OF THE ECLIPSE PUBLIC LICENSE
  * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE
  * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
  *
- * You can obtain a current copy of the Common Public License from
- * http://oss.software.ibm.com/developerworks/opensource/license-cpl.html
+ * You can obtain a current copy of the Eclipse Public License from
+ * http://www.opensource.org/licenses/eclipse-1.0.php
  *
  * Author:       Adrian Schuur <schuur@de.ibm.com>
  * Contributors: Viktor Mihajlovski <mihajlov@de.ibm.com>
  *               Markus Mueller <markus_mueller@de.ibm.com>
  *               Steve Shepherd <steve.shepherd@configuresoft.com>
- *               Heidi Neumann  <heidineu@de.ibm.com>
+ *               Heidi Eckhart  <heidineu@de.ibm.com>
  *               David Eger <dteger@us.ibm.com>
+ *               Tyrel Datwyler <tyreld@us.ibm.com>
  *
  * Description: Line command interface to DMTF conforming WBEM servers
 */
@@ -30,10 +33,6 @@
 #include <vector>
 #include <iostream>
 
-#if defined __GNUC__
-#include <ansidecl.h>
-#endif
-
 using namespace std;
 
 extern int dumpXml;
@@ -56,7 +55,7 @@ class StringArray : public vector<string> {
    StringArray(int n) : vector<string>(n) {}
    StringArray(int n, const string& s) : vector<string>(n,s) {}
    void add(const string& t) { push_back(t); }
-   void toStringBuffer(string &sb, char *sep=" ",char *q="") {
+   void toStringBuffer(string &sb, const char *sep=" ",const char *q="") {
      for (size_type n=0; n<size(); n++) {
        sb = sb + (*this).at(n) + q;
        if (n!=size()-1) sb += sep;
@@ -100,9 +99,9 @@ class XmlBuffer {
    int endTag(const char*, const XmlAttr *a=NULL);
    int attrsOk(const XmlElement&,XmlAttr&,const char* tag);
    string getValue(const char *v=NULL);
-   char *getContent(const XmlAttr*);
+   const char *getContent(const XmlAttr*);
    int skipElement(const char*);
-   int testAttr(char*,char*,int);
+   int testAttr(const char*,const char*,int);
 };
 
 class DoesNotExist {
@@ -130,13 +129,13 @@ template<class T> class ArrayXml : public vector<T> {
   public:
    ArrayXml() {}
    ArrayXml<T> * clone() const { return new ArrayXml<T>(*this);}
-   void add(const T& t) { push_back(t); }
+   void add(const T& t) { this->push_back(t); }
 #if !defined(GCC_VERSION) || GCC_VERSION >= 3000
-   T& operator[] (size_type n) {return at(n);}
+   T& operator[] (size_type n) {return this->at(n);}
    const T& operator[] (size_type n) const {return at(n);}
 #endif
    T *get(int n)    { return &(*this)[n]; }
-   void toStringBuffer(string &sb, char *sep=" ",char *q="") {
+   void toStringBuffer(string &sb, const char *sep=" ",const char *q="") {
      for (size_type n=0; n<this->size(); n++) {
        (*this).at(n).toStringBuffer(sb,q);
        if (n!=this->size()-1) sb += sep;
@@ -171,6 +170,7 @@ class QualifierXml;
 
 class ValueXml;
 class PropertyXml;
+class ParameterXml;
 class InstanceNameXml;
 class InstancePathXml;
 class InstanceXml;
@@ -183,12 +183,6 @@ class URL;
 class AValue;
 class ParamvalueXml;
 
-class MethodXml {
-  public:
-   string str;
-   MethodXml(XmlBuffer&);
-};
-
 class QualifierXml {
   public:
    string str;
@@ -198,6 +192,7 @@ class QualifierXml {
 
 typedef ArrayXml<QualifierXml>     QualifierArray;
 typedef ArrayXml<PropertyXml>      PropertyArray;
+typedef ArrayXml<ParameterXml>     ParameterArray;
 typedef ArrayXml<MethodXml>        MethodArray;
 typedef ArrayXml<ValueXml>         ValueArray;
 typedef ArrayXml<InstanceNameXml>  InstanceNameArray;
@@ -220,8 +215,9 @@ class AValue {
    StringArray vals;
    AValue(const char *a, const char *v) { attr=a; val=v; }
    void add(const char *v) {
-     vals.push_back(string(val));
-     val=""; 
+     if(vals.size()==0)
+       vals.push_back(string(val));
+     val="";
      vals.push_back(string(v)); }
 };
 
@@ -229,9 +225,12 @@ class AVPs {
    AValueArray avals;
    char type[256];
    char *all;
+   void parseRefList(char *name, char *s, bool isArray=false);
   public:
    AVPs(char*);
-   ~AVPs() { free(all); }
+   AVPs(char*,PropertyArray&);
+   AVPs(char*,ParameterArray&);
+   ~AVPs() { if(all) free(all); }
    static AVPs *makeAVPs(char *s, unsigned int n);
    AValue* get(unsigned int n);
    AValue* get(const char *a);
@@ -297,13 +296,14 @@ class ErrorXml : public XmlException {
 
 class KeyValueXml {
    static XmlElement elm;
+   static bool initialized;
   public:
    string valType, val;
    KeyValueXml(XmlBuffer&);
    KeyValueXml(const char*);
    KeyValueXml() {}
    KeyValueXml& operator=(const KeyValueXml& src) {if(this!=&src) {valType=src.valType; val = src.val;} return *this;}
-   void toStringBuffer(string&,char*);
+   void toStringBuffer(string&,const char*);
 };
 
 
@@ -313,7 +313,7 @@ class NameSpaceXml {
    string name;
    NameSpaceXml() {}
    NameSpaceXml(XmlBuffer&);
-   void toStringBuffer(string&,char*);
+   void toStringBuffer(string&,const char*);
 };
 
 class HostXml {
@@ -338,6 +338,7 @@ class NameSpacePathXml {
    LocalNameSpacePathXml lns;
    NameSpacePathXml() {}
    NameSpacePathXml(XmlBuffer&);
+   void toStringBuffer(string &sb, const char *q="");
 };
 
 class SubValueXml {
@@ -355,13 +356,7 @@ class SubValueXml {
 #define TypeSubValueInstancepath 4
 #define TypeSubValueClasspath 5
 #define TypeSubValueLocalClasspath 6
-   virtual void toStringBuffer(string &sb, char *q) { sb += ">>Not Supported>>"; }
-};
-
-class InstancenameValueXml : public SubValueXml {
-  public:
-   string instName;
-   InstancenameValueXml(XmlBuffer&);
+   virtual void toStringBuffer(string &sb, const char *q) { sb += ">>Not Supported>>"; }
 };
 
 class RetValXml {
@@ -389,7 +384,7 @@ class RetValXml {
    virtual RetValXml * clone() { return new RetValXml(*this); }
 #endif
    int getType() { return type; }
-   virtual void toStringBuffer(string &sb, char *q="") { sb += "** not implemented **"; }
+   virtual void toStringBuffer(string &sb, const char *q="") { sb += "** not implemented **"; }
 };
 
 class ValueXml : public RetValXml {
@@ -417,7 +412,7 @@ class ValueXml : public RetValXml {
    ValueXml * clone() { return new ValueXml(*this);}
    string getValue() { return val; }
    void toXmlBuffer(string &xb);
-   virtual void toStringBuffer(string&,char* q="");
+   virtual void toStringBuffer(string&,const char* q="");
 };
 
 class KeyBindingXml {
@@ -430,21 +425,22 @@ class KeyBindingXml {
    KeyBindingXml() : which(INVALID), name(0) {}
    KeyBindingXml(XmlBuffer&);
    void init(XmlBuffer&);
-   KeyBindingXml(char*,char*);
-   void toStringBuffer(string&,char*s="");
+   KeyBindingXml(const char*,const char*);
+   void toStringBuffer(string&,const char*s="");
 };
 
 
 class InstanceNameXml : public RetValXml {
    static XmlElement elm;
    string clsName;
+   string nspace;
   public:
    KeyBindingArray keys;
    InstanceNameXml() {}
    InstanceNameXml(XmlBuffer&, char op=0);
    InstanceNameXml * clone() { return new InstanceNameXml(*this);}
    void init(XmlBuffer&, char op);
-   virtual void toStringBuffer(string&,char *q="");
+   virtual void toStringBuffer(string&,const char *q="");
 };
 
 class InstanceNamesXml : public RetValXml, public InstanceNameArray {
@@ -458,13 +454,33 @@ class InstancepathValueXml : public SubValueXml {
   protected:
    static XmlElement elm;
   public:
+   string nspace;
    NameSpacePathXml nsp;
    InstanceNameXml name;
    InstancepathValueXml() {}
    InstancepathValueXml(XmlBuffer&);
    InstancepathValueXml * clone() { return new InstancepathValueXml(*this); }
    void init(XmlBuffer&);
-   virtual void toStringBuffer(string&,char*);
+   virtual void toStringBuffer(string&,const char*);
+};
+
+/*
+class InstancenameValueXml : public SubValueXml {
+  public:
+   string instName;
+   InstancenameValueXml(XmlBuffer&);
+};
+*/
+class InstancenameValueXml : public SubValueXml {
+  protected:
+   static XmlElement elm;
+  public:
+   InstanceNameXml name;
+   InstancenameValueXml() {}
+   InstancenameValueXml(XmlBuffer&);
+   InstancenameValueXml * clone() { return new InstancenameValueXml(*this); }
+   void init(XmlBuffer&);
+   virtual void toStringBuffer(string&,const char*);
 };
 
 class ClassnameValueXml : public SubValueXml {
@@ -482,7 +498,7 @@ class LocalInstancepathValueXml : public InstancepathValueXml {
    LocalInstancepathValueXml(XmlBuffer&);
    LocalInstancepathValueXml * clone() { return new LocalInstancepathValueXml(*this); }
    void init(XmlBuffer&);
-   virtual void toStringBuffer(string&,char*);
+   virtual void toStringBuffer(string&,const char*);
 };
 
 class PropertyXml {
@@ -507,10 +523,55 @@ class PropertyXml {
    PropertyXml(XmlBuffer&);
    PropertyXml(char*,int);
    void addQualifier(QualifierXml& q) { qualifiers.add(q); }
-   void toStringBuffer(string&,char*);
-   void toXmlBuffer(string&,AValue*,int vOnly=0);
+   void toStringBuffer(string&,const char*);
+   void toXmlBuffer(string&,AValue*,int vOnly=0,URL* url=NULL);
 };
 
+class ParameterXml {
+   static XmlElement elm,elmRef,elmArray, elmRefArray;
+   static bool initialized;
+   void initialize();
+   void init(XmlBuffer&);
+   void refToXmlBuffer(string &xb, const char *s, URL *url);
+  public:
+   string name, type, refCls, arraySize;
+   int pType;
+   string str;
+   int eprop, lprop;
+#define TypeParameter 1
+#define TypeParameterRef 2
+#define TypeParameterArray 3
+#define TypeParameterRefArray 4
+   QualifierArray qualifiers;
+   Option<ValueXml> value;
+   int key;
+   // so we can vector-ize PropertyXml's, we need a default constructor
+   ParameterXml() : pType(TypeNull) {}
+   ParameterXml(XmlBuffer&);
+   //   ParameterXml(char*,int);
+   void addQualifier(QualifierXml& q) { qualifiers.add(q); }
+   void toStringBuffer(string&,const char*);
+   void toXmlBuffer(string&,AValue*,int vOnly=0,URL* url=NULL);
+};
+
+class MethodXml {
+   static XmlElement elm;
+   static bool initialized;
+   void   init(XmlBuffer&);
+   QualifierArray qualifiers;
+   ParameterArray parameters;
+  public:
+   string name;
+   string str;
+   MethodXml() : name("") {}
+   MethodXml(XmlBuffer&);
+   //   MethodXml * clone()    { return new MethodXml(*this); }
+   QualifierArray &getQualifiers()        { return qualifiers; }
+   ParameterArray &getParameters()        { return parameters; }
+   void addQualifier(QualifierXml& q)     { qualifiers.add(q); }
+   void addParameter(ParameterXml& p)     { parameters.add(p); }
+   //   void toStringBuffer(string&,char*);
+};
 
 class ClassXml : public RetValXml {
    static XmlElement elm;
@@ -529,10 +590,11 @@ class ClassXml : public RetValXml {
    PropertyArray &getProperties()        { return properties;  }
    QualifierArray &getQualifiers()       { return qualifiers;  }
    PropertyArray &getKeys()              { return keys;        }
+   MethodArray &getMethods()             { return methods;     }
    void addQualifier(QualifierXml& q)    { qualifiers.add(q);  }
    void addProperty(PropertyXml& p)      { properties.add(p); if (p.key) keys.add(p); }
    void addMethod(MethodXml& m)          { methods.add(m);     }
-   virtual void toStringBuffer(string&,char *q="");
+   virtual void toStringBuffer(string&,const char *q="");
    int keysOk(URL&);
 };
 
@@ -562,7 +624,7 @@ class InstanceXml : public RetValXml {
    QualifierArray &getQualifiers()    { return qualifiers; }
    void addQualifier(QualifierXml& q) { qualifiers.add(q); }
    void addProperty(PropertyXml& p)   { properties.add(p); }
-   virtual void toStringBuffer(string&,char *p="");
+   virtual void toStringBuffer(string&,const char *p="");
 };
 
 class NamedInstanceXml : public RetValXml {
@@ -627,7 +689,7 @@ class ClassNameXml : public RetValXml {
    ClassNameXml(XmlBuffer&);
    ClassNameXml * clone() { return new ClassNameXml(*this);}
    void init(XmlBuffer&);
-   virtual void toStringBuffer(string &sb, char *q="");
+   virtual void toStringBuffer(string &sb, const char *q="");
 };
 
 
@@ -650,7 +712,7 @@ class ParamvalueXml : public RetValXml {
   ParamvalueXml(const ParamvalueXml&);
   ~ParamvalueXml() {delete val;}
   ParamvalueXml *clone() {return new ParamvalueXml(*this);}
-  virtual void toStringBuffer(string &sb, char *q="");
+  virtual void toStringBuffer(string &sb, const char *q="");
 };
 
 class IReturnvalueXml {
@@ -714,7 +776,7 @@ class MethodresponseXml : public RetValXml,  public ResponseXml {
    MethodresponseXml(const MethodresponseXml &);
    ~MethodresponseXml() {delete rvl; delete pvll; delete err;}
    MethodresponseXml *clone() {return new MethodresponseXml(*this);};
-   virtual void toStringBuffer(string&,char *q="");
+   virtual void toStringBuffer(string&,const char *q="");
    virtual RetValXml *getReturnData() {
      if (err) throw ErrorXml(*err);
      return clone();
@@ -759,9 +821,11 @@ class CimXml {
 
 
 class URL {
+   int getNetDeviceNum(string);
   public:
    string scheme;
    string host;
+   long zone_id;
    string port;
    string cName;
    string user;
@@ -773,6 +837,7 @@ class URL {
    int  refLookAhead(char*,char**);
    void toStringBuffer(string&);
    void toStringBuffer(string&,int nsOnly);
+   void toStringBufferGI(string&);
 };
 
 #define INCLUDEQUAL false
@@ -783,7 +848,7 @@ class Cimom {
   static char xmlTrailer[];
  protected:
   virtual bool supportsSSL() { return false; };
-  virtual void genRequest(URL&,char*,bool cls=false,bool keys=false)=0;
+  virtual void genRequest(URL&,const char*,bool cls=false,bool keys=false)=0;
   virtual void addPayload(const string& pl)=0;
   virtual string getResponse()=0;
   char *newstr;
