@@ -1,5 +1,5 @@
 /*
- * $Id: Cimghttp.cpp,v 1.8 2008/08/07 18:25:41 tyreld Exp $
+ * $Id: Cimghttp.cpp,v 1.9 2008/12/12 00:52:00 tyreld Exp $
  *
  * Cimghttp.cpp
  *
@@ -32,7 +32,7 @@ CimomGhttp::CimomGhttp()
    newstr=NULL;
 }
 
-void CimomGhttp::genRequest(URL &url, char *op, bool cls, bool keys)
+void CimomGhttp::genRequest(URL &url, const char *op, bool cls, bool keys)
 {
    if (!supportsSSL() && url.scheme == "https")
        throw HttpException("ghttp does not support https urls.");
