@@ -1,5 +1,5 @@
 /*
- * $Id: main.cpp,v 1.34 2009/06/24 17:44:48 tyreld Exp $
+ * $Id: main.cpp,v 1.36 2013/09/19 14:34:26 hellerda Exp $
  *
  * main.cpp
  *
@@ -61,14 +61,15 @@ extern int addType;
 extern int openwbem;
 extern int dlmRefs;
 extern int reqChunking;
-
+extern int expect100;
+int waitTime;
 
 
 static void usage(int indir, int op, char *cmd)
 {
    cerr<<"usage:";
    cerr<<"\twbemcli gc|gcd|dc|ec|ecn|gi|ci|mi|di|ei|ein|ain|ai|rin|ri|gp|sp|cm|cmx\n"
-         "\t        [-nl] [-dx] [-t] [-cte] [-h] [-v]\n"
+         "\t        [-nl] [-dx] [-t] [-cte] [-h] [-v] [-w <waittime>] [-e100]\n"
          "\t        [-noverify] [-cacert <file>] [-clientcert <file>] [-clientkey <file>]\n"
          "\t        [-ac <assocClass>] [-arc <resultClass>] [-ar <role>] [-arr <resultRole>]\n"
          "\t        objectPath [prop=value,[...]] [props[,...]]\n"<<endl;
@@ -117,7 +118,7 @@ static void cleanUpMsg(string &msg)
 
 int main (int argc, char *argv[]) {
    int pos, retval = 0;
-   char *cmd, *params, *opStr=NULL,*urlStr=NULL,*xtraStr=NULL;
+   char *cmd, *params, *endptr, *opStr=NULL,*urlStr=NULL,*xtraStr=NULL;
    char *assocClass=NULL,*resultClass=NULL,*role=NULL,*resultRole=NULL;
    const char *cacert=CACERT, *clientcert=NULL, *clientkey=NULL;
    int n=1,err=0,op=0,cmdln,help=0,indir=0, noverify=0;
@@ -200,8 +201,13 @@ int main (int argc, char *argv[]) {
  	   else if (strcmp(*argv,"-arc")==0) resultClass=*(++argv);
  	   else if (strcmp(*argv,"-ar")==0) role=*(++argv);
  	   else if (strcmp(*argv,"-arr")==0) resultRole=*(++argv);
+           else if (strcmp(*argv,"-e100")==0) expect100 = 1;
+           else if (strcmp(*argv,"-w")==0) {
+             waitTime = (int) strtol(*(++argv),&endptr,0);
+             if (endptr == *argv) err=1;  // no digits found
+           }
            else if (strcmp(*argv,"-v")==0) {
-              cerr<<"* wbemcli Version " PACKAGE_VERSION ", $Date: 2009/06/24 17:44:48 $" <<endl;
+              cerr<<"* wbemcli Version " PACKAGE_VERSION ", $Date: 2013/09/19 14:34:26 $" <<endl;
               goto fini;
            }
            else err=1;
